#!/bin/sh
#########################################################################
#                                                                        #
# (c) Copyright 2004, Cadence Design Systems, Inc.                       #
# All rights reserved.                                                   #
#                                                                        #
# This software is the proprietary information of Cadence Design         #
# Systems, Inc. and may not be copied or reproduced in whole or in part  #
# onto any medium without Cadence's express prior written consent.       #
#                                                                        #
# This software is provided to the end user solely for his/her use.  No  #
# warranties are expressed or implied herein including those as to       #
# merchantability and fitness for a particular purpose.  In no event     #
# shall Cadence be held liable for loss of profit, business              #
# interruption, data, loss of information, or any other pecuniary loss   #
# including but not limited to special, incidental, consequential, or    #
# other damages.                                                         #
#                                                                        #
########################################################################## 
echo "$0 Version: 2"
# 0: 01.01.2005 vincentr initial script                                  #
# 1: 04.07.2007 jmueller updated to IFV 6.1                              #
# 2: 05.07.2007 jmueller IPCM compliant naming conventions               #
########################################################################## 

#set -x

IFV=ifv
CLK=clk
RST=rst
RSTLO=FALSE
RTL=rtl
VHD=FALSE
ARCH=rtl
FILENAME=IFVDIR_UNSET
DATE=`date`
USER=`whoami`
SEPARATOR=.
OSNAME=`uname`

if [ $OSNAME = "Linux" ]
then
  alias echo="echo -e"
fi


#
# Standard Error Message
#
usage_msg="\nUsage:  $0 [options] <top_block_name>\n\n"
usage_msg=$usage_msg"   Option:\t Default:\t Description:\n"
usage_msg="$usage_msg   ------------\t --------\t ------------\n"
usage_msg="$usage_msg   -dir  <name>\t ifv\t\t name of directory to create\n"
usage_msg="$usage_msg   -clk  <name>\t clk\t\t clock name\n"
usage_msg="$usage_msg   -rst  <name>\t rst\t\t reset name\n"
usage_msg="$usage_msg   -rstlo\t\t *unset*\t if reset is active low\n"
usage_msg="$usage_msg   -rtl  <path>\t rtl\t\t relative path to rtl directory from current location\n"
usage_msg="$usage_msg   -vhd\t\t *unset*\t set if top level is VHDL, Verilog assumed when unset\n"
usage_msg="$usage_msg   -arch <name>\t rtl\t\t architecture name for vhdl top level\n"
usage_msg="$usage_msg   -file <name>\t *default*\t set to HDL filename if different to design name\n"


#
# Check basic usage
#

if [ $# -lt 1 ]; then
  echo $usage_msg
  exit 127
fi

#
# Get inputs
# 

while [ $# -gt 1 ]; do
#echo "start# $#"
  case $1 in
    -d*)   #echo "-d*"
           shift
           IFV=$1
           ;;
    -c*)   #echo "-c*"
           shift
           CLK=$1
           ;;
    -rstl*) #echo "-rstl*"
           RSTLO=TRUE
           ;;
    -rs*)  #echo "-rs*"
           shift
           RST=$1
           ;;
    -rt*) #echo "-rt*"
           shift
           RTL=$1
           ;;
    -vh*) #echo "-vh*"
           VHD=TRUE
           SEPARATOR=:
           ;;
    -a*) #echo "-a*"
           shift
           ARCH=$1
           ;;
    -f*) #echo "-f*"
           shift
           FILENAME=$1
           ;;
    *) #echo "*"
           echo $usage_msg
           exit 127
           ;;
  esac
  shift
#echo "end# $#"
done

if [ $# -lt 1 ]; then
  echo $usage_msg
  exit 127
fi

BLOCK=$1
shift

case $BLOCK in
  -*) 
     echo $usage_msg
     exit 127
     ;;
  esac

#
# test for existance of directory
#

if [ -d $IFV ]
then
  echo "$0: Error: Target directory '$IFV' already exists"
  exit 127
else
  mkdir $IFV
fi

#
# Set filename to default if not set by user
# 

if [ $FILENAME = "IFVDIR_UNSET" ]
then
  if [ $VHD = "TRUE" ]
  then
    FILENAME=$BLOCK.vhd
  else
    FILENAME=$BLOCK.v
  fi
fi

#
# create dummy file and directory if it does not exist
# 

if [ -d $RTL ]
then
  echo "$0: Note: Target directory '$RTL' already exists"
else
  mkdir $RTL
  if [ $VHD = "TRUE" ]
  then
    echo "entity $BLOCK is end $BLOCK;\narchitecture $ARCH of $BLOCK is begin end $ARCH;\n" > $RTL/$FILENAME
  else
    echo "module $BLOCK; endmodule\n" > $RTL/$FILENAME
  fi
fi


#
# Test Input
#
#
# echo "CLK      = $CLK"
# echo "RST      = $RST"
# echo "RSTLO    = $RSTLO"
# echo "RTL      = $RTL"
# echo "VHD      = $VHD"
# echo "ARCH     = $ARCH"
# echo "FILENAME = $FILENAME"


#
# Compile ifv_$BLOCK.f file text
#

f_txt="// Auto Generated file: ifv_$BLOCK.f\n"
f_txt=$f_txt"// Created:             $DATE\n"
f_txt=$f_txt"// Created By:          $USER\n"
f_txt=$f_txt"//\n"
f_txt=$f_txt"// To run IFV in batch mode: ifv -f ifv_$BLOCK.f \n"
f_txt=$f_txt"// To run IFV with GUI:      ifv -f ifv_$BLOCK.f +gui\n"
f_txt=$f_txt"\n"
f_txt=$f_txt"// Performance Tuning Switches\n"
f_txt=$f_txt"//+disable_afa\n"
f_txt=$f_txt"//+formalbuildargs++nohal\n"
f_txt=$f_txt"//+disable_deadcode\n"
f_txt=$f_txt"//+disable_conditional_deadcode\n"
f_txt=$f_txt"//+disable_fsm\n"
f_txt=$f_txt"//+disable_bus\n"
f_txt=$f_txt"//+disable_xcheck\n"
f_txt=$f_txt"//+noassert_synth_pragma\n"
f_txt=$f_txt"\n"

if [ $VHD = "TRUE" ]
then
  f_txt=$f_txt"// Set generic ABV_ON to TRUE\n"
  f_txt=$f_txt"+generic+\"ABV_ON => \\\"TRUE\\\"\"\n"
  f_txt=$f_txt"\n"
  f_txt=$f_txt"// User Generics here:\n"
else
  f_txt=$f_txt"// Turn the ABV_ON macro on\n"
  f_txt=$f_txt"+define+ABV_ON\n"
  f_txt=$f_txt"\n"
  f_txt=$f_txt"// User Macros here:\n"
fi
#
f_txt=$f_txt"\n"
f_txt=$f_txt"\n"
#
if [ $VHD = "TRUE" ]
then
  f_txt=$f_txt"// top level entity_name:architecture_name\n"
  f_txt=$f_txt"+top+$BLOCK:$ARCH\n"
else 
  f_txt=$f_txt"// include directories for resolving \`include directives\n"
  f_txt=$f_txt"+incdir+../$RTL\n"
fi
#
f_txt=$f_txt"\n"
f_txt=$f_txt"// RTL design files\n"
f_txt=$f_txt"../$RTL/$FILENAME\n"
f_txt=$f_txt"\n"
f_txt=$f_txt"// Tcl command file\n"
f_txt=$f_txt"+tcl+run.tcl\n"
f_txt=$f_txt"\n"
f_txt=$f_txt"//+rangeoverflow\n"


#
# Compile $BLOCK.tcl file text
#

tcl_txt="########################################\n"
tcl_txt=$tcl_txt"# Auto Generated file: run.tcl\n"
tcl_txt=$tcl_txt"# Created:             $DATE\n"
tcl_txt=$tcl_txt"# Created By:          $USER\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"# For Interactive Incremental Iteration\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"reset\n"
tcl_txt=$tcl_txt"alias . input run.tcl\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"# Session Start\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"date\n"
tcl_txt=$tcl_txt"uname -a\n"
tcl_txt=$tcl_txt"whoami\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"# Tool settigs\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"#define vacuity off\n"
tcl_txt=$tcl_txt"#define modeling off\n"
tcl_txt=$tcl_txt"#define witness_check trace\n"
tcl_txt=$tcl_txt"#define debugmode off\n"
tcl_txt=$tcl_txt"#define effort low\n"
tcl_txt=$tcl_txt"#define halo seq\n"
tcl_txt=$tcl_txt"#define engine sword\n"
tcl_txt=$tcl_txt"#define _nextGenIfv on\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"# Clock specification\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"clock -add $CLK\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"# Initialization\n"
tcl_txt=$tcl_txt"########################################\n"
#
if [ $RSTLO = "TRUE" ]
then
  tcl_txt=$tcl_txt"force $RST  0\n"
else
  tcl_txt=$tcl_txt"force $RST  1\n"
fi
#
tcl_txt=$tcl_txt"run 4\n"
tcl_txt=$tcl_txt"init -load -current\n"
tcl_txt=$tcl_txt"init -show\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"# Deactivate reset for formal analysis\n"
tcl_txt=$tcl_txt"########################################\n"
#
if [ $RSTLO = "TRUE" ]
then
  tcl_txt=$tcl_txt"constraint -add -pin $RST 1 -reset\n"
else
  tcl_txt=$tcl_txt"constraint -add -pin $RST 0 -reset\n"
fi
#
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"# Activate Constraints\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"assertion -delete -all\n"
tcl_txt=$tcl_txt"# Delete lower level constraints\n"
tcl_txt=$tcl_txt"constraint -delete *"$SEPARATOR"input_* -r\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"# Turn former lower level constraints into assertions\n"
tcl_txt=$tcl_txt"#assertion -add *"$SEPARATOR"input_* -r\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"# Add top level constraints\n"
tcl_txt=$tcl_txt"constrain -add *input_*\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"# Show resulting constraints\n"
tcl_txt=$tcl_txt"constrain -show\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"# Design Info\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"scope\n"
tcl_txt=$tcl_txt"report -design\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"# Automatic RTL Checks\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"check -show DESIGN\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"# Automatic Modeling Checks\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"prove\n"
tcl_txt=$tcl_txt"check -show MODELING\n"
tcl_txt=$tcl_txt"date\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"# Automatic Pragma Checks\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"assertion -add -pragma\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"# Automatic Deadcode Checks\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"assertion -add -deadcode\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"# Automatic FSM Checks\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"assertion -add -fsm\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"# Automatic Bus Checks\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"assertion -add -bus\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"# Automatic X Checks\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"assertion -add -xcheck\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"# Automatic Range Overflow Checks\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"assertion -add -rangeoverflow\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"# User Cover Checks\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"assertion -add -cover\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"# User Assertion Checks\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"assertion -add -specification\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"# Run the prove\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"prove"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"# Report formal results\n"
tcl_txt=$tcl_txt"########################################\n"
tcl_txt=$tcl_txt"#assert -show -time\n"
tcl_txt=$tcl_txt"assert -sum -time\n"

clean_txt="rm -rf *.log *.key *.shm *.elog *.ecov  *~ hal.design_facts SSout INCA* .ifv* .nc* .vhdl* .lock* .formal* .reinvoke* .simvision"
echo -e $f_txt > $IFV/ifv_$BLOCK.f
echo -e $tcl_txt > $IFV/run.tcl
echo -e $clean_txt > $IFV/CLEAN
chmod a+x $IFV/CLEAN

echo -e ""
echo -e "$0: IFV Working Directory '$IFV' created and populated"
echo -e "$0: 'cd $IFV' and use 'ifv -f ifv_$BLOCK.f [+gui]' to run."
echo -e ""

exit 0


